#ifndef MAIN_H
#define MAIN_H

#include <Ecore.h>
#include <Evas.h>
#include <Ecore_Evas.h>
#include <Ecore_Fb.h>
#include <Edje.h>
#include <Eet.h>
#include <math.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>

#define IM "data/images/"
#define PT "data/pointers/"
#define FN "data/fonts/"
#define ED "data/edjes/"
#define WP "data/wallpapers/"

/* global variables */
extern double       start_time;
extern double       fps;
extern Ecore_Evas  *ee;
extern Evas        *evas;

/* globally accessible functions */
/*********/  /********************/  /****************************************/
void         calibrate_pos_set       (int pos);
int          calibrate_pos_get       (void);
void         calibrate_finish        (void);
void         calibrate_start         (void (*func_after) (void));

/*********/  /********************/  /****************************************/
void         bg_resize               (double w, double h);
void         bg_start                (void);
void         bg_go                   (void);
void         bg_file_set             (char *file);
    
/*********/  /********************/  /****************************************/
int          app_start               (int argc, const char **argv);
void         app_finish              (void);

/*********/  /********************/  /****************************************/
void         menus_start             (void);
void         menus_main              (void);
void         menus_settings          (void);
void         menus_utilities         (void);
    
/*********/  /********************/  /****************************************/
void         menu_push               (void);
void         menu_pop                (void);
void         menu_go                 (void);
void         menu_ungo               (void);
void         menu_item_add           (char *label,
				      char *icon, 
				      void (*func) (int n));
void         menu_item_select_set    (int n);
int          menu_item_select_get    (void);
    
/*********/  /********************/  /****************************************/
Evas_Object *box_add                 (Evas *evas);
int          box_freeze              (Evas_Object *obj);
int          box_thaw                (Evas_Object *obj);
void         box_orientation_set     (Evas_Object *obj, int horizontal);
int          box_pack_start          (Evas_Object *obj, Evas_Object *child);
int          box_pack_end            (Evas_Object *obj, Evas_Object *child);
int          box_pack_count_get      (Evas_Object *obj);
Evas_Object *box_pack_object_nth     (Evas_Object *obj, int n);
void         box_unpack              (Evas_Object *obj);
void         box_pack_info_set       (Evas_Object *obj, 
				      Evas_Coord minw, 
				      Evas_Coord minh);
void         box_min_size_get        (Evas_Object *obj, 
				      Evas_Coord *minw, 
				      Evas_Coord *minh);
void         box_max_size_get        (Evas_Object *obj, 
				      Evas_Coord *maxw, 
				      Evas_Coord *maxh);
void         box_select_func_set     (Evas_Object *obj, 
				      void (*sel_func) (Evas_Object *obj), 
				      void (*unsel_func) (Evas_Object *obj));
void         box_select_set          (Evas_Object *obj, int n);
int          box_select_get          (Evas_Object *obj);
void         box_select_time_set     (Evas_Object *obj, double sec);
double       box_select_time_get     (Evas_Object *obj);

/*********/  /********************/  /****************************************/
Evas_Object *icon_add                (Evas *evas);
void         icon_file_set           (Evas_Object *obj, const char *file);
const char  *icon_file_get           (Evas_Object *obj);
void         icon_smooth_scale_set   (Evas_Object *obj, int smooth);
int          icon_smooth_scale_get   (Evas_Object *obj);
void         icon_size_get           (Evas_Object *obj, int *w, int *h);
int          icon_fill_inside_get    (Evas_Object *obj);
void         icon_fill_inside_set    (Evas_Object *obj, int fill_inside);

/*********/  /********************/  /****************************************/
void         screen_push             (void (*start) (void *scr_data),
				      void (*stop)  (void *scr_data),
				      void (*go)    (void *scr_data),
				      void (*ungo)  (void *scr_data),
				      void *scr_data);
void         screen_pop              (void);
void         screen_go               (void);
void         screen_ungo             (void);
    
/*********/  /********************/  /****************************************/
void         wallpaper_start         (void);

/*********/  /********************/  /****************************************/
Evas_Object *list_add                (Evas *evas);
int          list_freeze             (Evas_Object *obj);
int          list_thaw               (Evas_Object *obj);
void         list_orientation_set    (Evas_Object *obj, int horizontal);
int          list_pack_start         (Evas_Object *obj, Evas_Object *child);
int          list_pack_end           (Evas_Object *obj, Evas_Object *child);
int          list_pack_count_get     (Evas_Object *obj);
Evas_Object *list_pack_object_nth    (Evas_Object *obj, int n);
void         list_unpack             (Evas_Object *obj);
void         list_pack_info_set      (Evas_Object *obj, 
				      Evas_Coord minw, 
				      Evas_Coord minh);
void         list_min_size_get       (Evas_Object *obj, 
				      Evas_Coord *minw, 
				      Evas_Coord *minh);
void         list_max_size_get       (Evas_Object *obj, 
				      Evas_Coord *maxw, 
				      Evas_Coord *maxh);
void         list_select_func_set    (Evas_Object *obj, 
				      void (*sel_func) (Evas_Object *obj), 
				      void (*unsel_func) (Evas_Object *obj));
void         list_select_set         (Evas_Object *obj, int n);
int          list_select_get         (Evas_Object *obj);
void         list_select_time_set    (Evas_Object *obj, double sec);
double       list_select_time_get    (Evas_Object *obj);

/*********/  /********************/  /****************************************/
time_t       file_mod_time           (char *file);
int          file_exists             (char *file);
int          file_is_dir             (char *file);
int          file_mkdir              (char *dir);
int          file_mkpath             (char *path);
int          file_cp                 (char *src, char *dst);
char        *file_realpath           (char *file);
char        *file_get_file           (char *path);
char        *file_get_file_no_ext    (char *path);
char        *file_get_dir            (char *file);
int          file_can_exec           (char *file);
char        *file_readlink           (char *link);
Evas_List   *file_ls                 (char *dir);
void         file_ls_free            (Evas_List *ls);

#endif
