#include "main.h"

typedef struct _Menu_Item Menu_Item;
typedef struct _Menu Menu;

struct _Menu
{
   unsigned char deleted : 1;
   unsigned char shown : 1;
  
   Evas_Object *o_menu;
   Evas_Object *o_menu_box;
   Evas_Object *o_menu_icon;
   Evas_List *items_menu;
   
   int selection;
};

struct _Menu_Item
{
   char *label;
   char *icon_file;
   Evas_Object *edje;
   Evas_Object *icon;
   Evas_Object *icon2;
   void (*func) (int n);
};

static Evas_List *menus = NULL;

static void _menu_cb_item_select(Evas_Object *o);
static void _menu_cb_item_unselect(Evas_Object *o);
static void _menu_cb_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
static void _menu_cb_stop_done(void *data, Evas_Object *o, const char *emission, const char *source);

void
menu_push(void)
{
   Menu *m;
   
   if (menus)
     {
	m = menus->data;
	edje_object_signal_emit(m->o_menu, "stop", "");
     }
   
   m = calloc(1, sizeof(Menu));
   
   menus = evas_list_prepend(menus, m);
}

void
menu_pop(void)
{
   Menu *m;
   
   if (!menus) return;
   m = menus->data;
   m->deleted = 1;
   m->shown = 0;
   if (m->o_menu)
     {
	edje_object_signal_emit(m->o_menu, "stop", "");
     }
   menus = evas_list_remove(menus, m);
   
   if (menus)
     menu_go();
   else
     screen_pop();
}

void
menu_go(void)
{
   Menu *m;
   Evas_List *l;
   
   if (!menus) return;
   m = menus->data;
   m->shown = 1;
   if (!m->o_menu)
     {
	Evas_Coord w, h;
	Evas_Object *o;
	
	evas_output_viewport_get(evas, NULL, NULL, &w, &h);
	
	o = edje_object_add(evas);
	edje_object_file_set(o, ED"default.edj", "menu/menu");
	evas_object_move(o, 0, 0);
	evas_object_resize(o, w, h);
	evas_object_event_callback_add(o, EVAS_CALLBACK_KEY_DOWN, _menu_cb_key_down, m);
	edje_object_signal_callback_add(o, "stop", "done", _menu_cb_stop_done, m);
	m->o_menu = o;
	
	o = box_add(evas);
	box_select_func_set(o, _menu_cb_item_select, _menu_cb_item_unselect);
	edje_object_part_swallow(m->o_menu, "menu", o);
	evas_object_show(o);
	m->o_menu_box = o;
	
	o = icon_add(evas);
	icon_file_set(o, "");
	icon_fill_inside_set(o, 1);
	edje_object_part_swallow(m->o_menu, "icon", o);
	evas_object_show(o);
	m->o_menu_icon = o;
	
	box_freeze(m->o_menu_box);
	for (l = m->items_menu; l; l = l->next)
	  {
	     Menu_Item *mi;
	     
	     mi = l->data;
	     
	     o = edje_object_add(evas);
	     edje_object_file_set(o, ED"default.edj", "menu/item");
	     edje_object_part_text_set(o, "label", mi->label);
	     box_pack_info_set(o, 128, 24);
	     box_pack_end(m->o_menu_box, o);
	     mi->edje = o;
	     
	     o = icon_add(evas);
	     icon_file_set(o, mi->icon_file);
	     icon_fill_inside_set(o, 1);
	     edje_object_part_swallow(mi->edje, "icon", o);
	     mi->icon = o;
	     
	     if (edje_object_part_exists(mi->edje, "icon2"))
	       {
		  o = icon_add(evas);
		  icon_file_set(o, mi->icon_file);
		  icon_fill_inside_set(o, 1);
		  edje_object_part_swallow(mi->edje, "icon2", o);
		  mi->icon2 = o;
	       }
	  }
	box_thaw(m->o_menu_box);
     }

   box_select_time_set(m->o_menu_box, 0.0);
   menu_item_select_set(m->selection);
   menu_item_select_set(m->selection);
   box_select_time_set(m->o_menu_box, 0.5);
   
   evas_object_show(m->o_menu);
   edje_object_signal_emit(m->o_menu, "start", "");
   evas_object_focus_set(m->o_menu, 1);
   
   for (l = m->items_menu; l; l = l->next)
     {
	Menu_Item *mi;
	
	mi = l->data;
	evas_object_show(mi->edje);
	evas_object_show(mi->icon);
	if (mi->icon2) evas_object_show(mi->icon2);
     }
}

void
menu_ungo(void)
{
   Menu *m;
   
   if (!menus) return;
   m = menus->data;
   m->shown = 0;
   edje_object_signal_emit(m->o_menu, "stop", "");
}

void
menu_item_add(char *label, char *icon, void (*func) (int n))
{
   Menu_Item *mi;
   Evas_Object *o;
   Menu *m;
   
   if (!menus) return;
   m = menus->data;   
   mi = calloc(1, sizeof(Menu_Item));
   
   if (label) mi->label = strdup(label);
   else mi->label = strdup("");
   if (icon) mi->icon_file = strdup(icon);
   else mi->icon_file = strdup("");
   mi->func = func;
   
   m->items_menu = evas_list_append(m->items_menu, mi);
}

void
menu_item_select_set(int n)
{
   int nn;
   Menu_Item *mi;
   Menu *m;
   
   if (!menus) return;
   m = menus->data;
   nn = evas_list_count(m->items_menu);
   m->selection = n;
   if (n < 0) n += nn;
   else if (n >= nn) n -= nn;
   m->selection = n;
   if (m->o_menu_box)
     {
	box_select_set(m->o_menu_box, m->selection);
     }
   mi = evas_list_nth(m->items_menu, m->selection);
   if (m->o_menu_icon)
     {
	icon_file_set(m->o_menu_icon, mi->icon_file);
     }
}

int
menu_item_select_get(void)
{
   Menu *m;
   
   if (!menus) return;
   m = menus->data;
   return m->selection;
}

static void
_menu_cb_item_select(Evas_Object *o)
{
   Menu *m;
   
   if (!menus) return;
   m = menus->data;
   if (m->deleted) return;
   if (m->o_menu)
     {
	edje_object_signal_emit(o, "select", "item");
	edje_object_signal_emit(m->o_menu, "select", "item");
     }
}

static void
_menu_cb_item_unselect(Evas_Object *o)
{
   Menu *m;
   
   if (!menus) return;
   m = menus->data;
   if (m->deleted) return;
   if (m->o_menu)
     {
	edje_object_signal_emit(o, "unselect", "item");
     }
}

/* NB: on ipaq
 *
 * "Escape"
 *
 * "F1"   "F2"    "F3"   "F4"
 *            "Up"
 *   "Left" "Return" "Right"
 *           "Down"
 * 
 */
static void
_menu_cb_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
   Evas_Event_Key_Down *ev;
   Menu *m;
   
   if (!menus) return;
   m = menus->data;
   ev = event_info;
   if (!strcmp(ev->keyname, "Up"))
     {
	int i;
	
	i = menu_item_select_get();
	i--;
	menu_item_select_set(i);
     }
   else if (!strcmp(ev->keyname, "Down"))
     {
	int i;
	
	i = menu_item_select_get();
	i++;
	menu_item_select_set(i);
     }
   else if ((!strcmp(ev->keyname, "Left")) ||
	    (!strcmp(ev->keyname, "Escape")))
     {
	menu_pop();
     }
   else if ((!strcmp(ev->keyname, "Right")) ||
	    (!strcmp(ev->keyname, "Return")))
     {
	int i;
	Menu_Item *mi;
	
	i = menu_item_select_get();
	mi = evas_list_nth(m->items_menu, i);
	if (mi->func) mi->func(i);
     }
}

static void
_menu_cb_stop_done(void *data, Evas_Object *o, const char *emission, const char *source)
{
   Menu *m;
   Evas_List *l;
   
   m = data;
   
   evas_object_hide(m->o_menu);
   
   if (m->shown) return;
   
   for (l = m->items_menu; l; l = l->next)
     {
	Evas_Object *o;
	Menu_Item *mi;
	
	mi = l->data;
	evas_object_del(mi->icon);
	if (mi->icon2) evas_object_del(mi->icon2);
	evas_object_del(mi->edje);
	mi->icon = NULL;
	mi->icon2 = NULL;
	mi->edje = NULL;
     }
   evas_object_del(m->o_menu_box);
   evas_object_del(m->o_menu_icon);
   evas_object_del(m->o_menu);
   m->o_menu_box = NULL;
   m->o_menu_icon = NULL;
   m->o_menu = NULL;

   if (!m->deleted) return;

   while (m->items_menu)
     {
	Evas_Object *o;
	Menu_Item *mi;
	
	mi = m->items_menu->data;
	if (mi->label) free(mi->label);
	if (mi->icon_file) free(mi->icon_file);
	free(mi);
	m->items_menu = evas_list_remove(m->items_menu, mi);
     }
   
   free(m);
}
